package field

import (
	"context"
	"math"
	"reflect"
	"testing"
)

func TestNumberField_Decode(t *testing.T) {
	tests := []struct {
		name    string
		field   *Field
		data    interface{}
		want    interface{}
		wantErr bool
	}{
		{"Correct", Number("int"), int64(2), int64(2), false},                               // #0
		{"Correct", Number("int"), 2.2, int64(2), false},                                    // #1
		{"Correct", Number("int"), 2, int64(2), false},                                      // #2
		{"Correct", Number("int"), float32(2.2), int64(2), false},                           // #3
		{"Correct", Number("int"), float64(2.6), int64(3), false},                           // #4
		{"Correct", Number("int"), 2.6, int64(3), false},                                    // #5
		{"MaxInt64", Number(NumberFormatInt), int64(math.MaxInt64), nil, true},              // #6
		{"MinInt64", Number(NumberFormatInt), int64(math.MinInt64), nil, true},              // #7
		{"maxInt in float", Number(NumberFormatInt), float64(maxInt), int64(maxInt), false}, // #8
		{"minInt in float", Number(NumberFormatInt), float64(minInt), int64(minInt), false}, // #9
		{"Convert error", Number(NumberFormatInt), math.MaxFloat64, nil, true},              // #10
		{"Convert error", Number(NumberFormatInt), -math.MaxFloat64, nil, true},             // #11
		{"Convert error", Number(NumberFormatInt), float64(math.MaxInt64), nil, true},       // #13
		{"Convert error", Number(NumberFormatInt), float64(math.MinInt64), nil, true},       // #14

		{"Correct", Number("float"), int8(2), 2.0, false},  // #15
		{"Correct", Number("float"), 2.2, 2.2, false},      // #16
		{"Correct", Number("float"), 2, 2.0, false},        // #17
		{"Correct", Number("float"), int64(2), 2.0, false}, // #18

		{"Wrong data", Number("int"), "", nil, true},         // #0
		{"Wrong data", Number("int"), []byte(""), nil, true}, // #1

		{"Wrong data", Number("float"), "", nil, true},         // #2
		{"Wrong data", Number("float"), []byte(""), nil, true}, // #3
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := Decode(context.Background(), tt.field, tt.data)
			if (err != nil) != tt.wantErr {
				t.Errorf("Decode() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("Decode() got = %v, want %v", got, tt.want)
			}
		})
	}
}

func TestNumberField_Encode(t *testing.T) {
	tests := []struct {
		name    string
		field   *Field
		data    interface{}
		want    interface{}
		wantErr bool
	}{
		{"Correct", Number("int"), int64(2), int64(2), false},     // #0
		{"Correct", Number("int"), 2.2, int64(2), false},          // #1
		{"Correct", Number("int"), 2, int64(2), false},            // #2
		{"Correct", Number("int"), float32(2.2), int64(2), false}, // #3
		{"Correct", Number("int"), float32(2.6), int64(3), false}, // #4
		{"Correct", Number("int"), 2.6, int64(3), false},          // #5

		{"Correct", Number("float"), int8(2), 2.0, false},  // #6
		{"Correct", Number("float"), 2.2, 2.2, false},      // #7
		{"Correct", Number("float"), 2, 2.0, false},        // #8
		{"Correct", Number("float"), int64(2), 2.0, false}, // #9

		{"Wrong data", Number("int"), "", nil, true},         // #0
		{"Wrong data", Number("int"), []byte(""), nil, true}, // #1

		{"Wrong data", Number("float"), "", nil, true},         // #2
		{"Wrong data", Number("float"), []byte(""), nil, true}, // #3
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := Encode(context.Background(), tt.field, tt.data)
			if (err != nil) != tt.wantErr {
				t.Errorf("Decode() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("Decode() got = %v, want %v", got, tt.want)
			}
		})
	}
}
